/*--------------------------skip 001.c  -----------------------------------
 * 
 * 
 * Hardware: ATTyne 85 mit 8MHz internal Clock
 * Board: skip 090-F and 090-B 
 * 
 * 
 * avrdude -p t85 -c avrisp2                      		   ; Console
 * avrdude: Device signature = 0x1e930b (probably t85)   ; default output
 * avrdude: safemode: Fuses OK (E:FF, H:DF, L:62)  			 ; default output
 * 
 * 
 * 
 * Fuse setting: Set low fuse for 8MHz internal clock
 * avrdude -p t85 -c avrisp2 -U lfuse:w:0xe2:m
 * 
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=attiny85 skip_001.c -o skip_001.elf
 * avr-objcopy -O ihex -j .text -j .data skip_001.elf skip.hex
 * avrdude -p attiny85 -c avrisp2 -U flash:w:skip_001.hex
 * 
 * 
 * PB0: Gate A Out ; MOSI
 * PB1: Gate B Out ; MISO
 * PB2: ADC1 ; Voltage input ; SCK
 * PB3: Gate input
 * PB4: Gate LED
 * PB5: Reset
 * 
 * 21.12.08
 * Start
 * 
 * ---------------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <stdlib.h>
#include <stdbool.h>

/* ------------ Variables --------------------------*/

volatile uint8_t control_voltage;							// Holds result from ADC1
const uint8_t upper_bound = 235;
const uint8_t lower_bound = 10;


/* ADC init */

void ADC_Init() 
{
 
  uint16_t result;
 
  ADMUX |= (0 << REFS0);					// reference voltage on AVCC    
	ADMUX |= (0 << REFS1);					// reference voltage on AVCC
	ADMUX |= (1 << ADLAR);					// left adjust result, return only 8bits
	ADMUX |= (1 << MUX0);						// using ADC1
  // Bit ADFR ("free running") in ADCSRA is already set to zero on startup; single conversion
  ADCSRA |= (1<<ADPS2);    				// Prescaler 8MHz / 16 = 250kHz rec: 50 - 200 kHz
  ADCSRA |= (1<<ADEN);            // ADC activate
  ADCSRA |= (1<<ADSC);						// eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC))			// Wait for end of conversion
	{         	
  }
  result = ADCW;									//ADCW must be read once
}


/* ADC single measure */
uint8_t ADC_Read()
{
  ADCSRA |= (1<<ADSC);            // single conversion
  while (ADCSRA & (1<<ADSC) ) 		// Wait for end of conversion
	{   
  }
  return (ADCH);           		    // ADC High byte read and return
}                             
  
void init_ports()
{
	DDRB &=~ (1 << PB2);         	  // set PB2 as input for ADC
	DDRB &=~ (1 << PB3);         	  // set PB3 as input for Gate/Trigger  
	DDRB |= (1 << PB0);							// set PB0 as output for A out
	DDRB |= (1 << PB1);							// set PB1 as output for B out
	DDRB |= (1 << PB4);							// set PB4 as output for Gate LED
}

volatile uint8_t set_random()
{
	return(rand() & 0xFF);				// Reduce the rang of rand() to 0..255
}

/* ----------------------MAIN ---------------------------*/


int main (void)
{					
	volatile uint8_t probal;
	volatile uint8_t control_voltage;
	
	init_ports();
	ADC_Init();
	probal = set_random();					// first set probal
	control_voltage = ADC_Read();		// first set of control_voltage
	
  while (1)												// for ever
	{

		volatile bool GateState = PINB & (1 << PB3);			// read the Gate state
		
		if (GateState)	// if the Gate is HIGH, turn LED on
		{
			PORTB |= (1 << PB4);				// write to PORTB register to set the Gate LED state to HIGH
			if (control_voltage < lower_bound)// 
				PORTB |= (1 << PB0);			// write to PORTB register to set the output A to HIGH
			if (control_voltage > upper_bound) 
				PORTB |= (1 << PB1);			// write to PORTB register to set the output B to HIGH
			if ((control_voltage <= upper_bound) && (control_voltage >= lower_bound))
			{
				if (control_voltage < probal)
				{
					PORTB |= (1 << PB0);			// write to PORTB register to set the output A to HIGH
				}
				else
				{
					PORTB |= (1 << PB1);			// write to PORTB register to set the output B to HIGH
				}
			}
		} // end if
		else 						// if the Gate is LOW, turn Gate LED off, all outputs to low
		{
			PORTB &= ~(1 << PB4);				// write to PORTB register to set the LED state to LOW
			PORTB &= ~(1 << PB0);				// write to PORTB register to set the output A to LOW
			PORTB &= ~(1 << PB1);				// write to PORTB register to set the output A to LOW
			probal = set_random();				// 0..255
			control_voltage = ADC_Read();						// 0..255	
		} //end else
	}	// end while
}	// end main

